﻿#include "stdafx.h"
#include "UpgradeInstallModule.h"
#include "AgentDLL.h"
#include "util.h"
#include <atlcomtime.h>
#include <memory>
#include "const.h"

const TCHAR *TIMEOUT_MESSAGE=_T("설치 작업 시간이 너무 오래 걸려서 실패했습니다.");

BOOL RunUpgrader( const TCHAR *guid, int execute_option, int *error_code, CString *error_msg )
{
	AgentDLLWrapper *agentdll = AgentDLLManager::Find(guid);
	if( NULL == agentdll )
	{
		return FALSE;
	}

	std::auto_ptr<AgentDLLWrapper> agent(agentdll);
	if( FALSE == agent->RunUpgrader(execute_option, error_code, error_msg ) )
	{
		return FALSE;
	}

	CALLBACK_FINISHED_STATUS ret = CALLBACK_FINISHED_ERROR;
	BOOL bContinue = TRUE;
	int status;

	COleDateTime limitTime = COleDateTime::GetCurrentTime();
	limitTime += COleDateTimeSpan( 0, 0, INSTALL_TIMEOUT_MINUTE, 0 );

	while( bContinue )
	{
		if( agent->GetUpdateStatus(status, error_code, error_msg ) == FALSE )
			return FALSE;

		COleDateTime curTime = COleDateTime::GetCurrentTime();
		if( curTime >= limitTime )
		{
			status = NUPDATE_STATUS_ERROR;
			*error_msg = TIMEOUT_MESSAGE;
			*error_code= NUPDATE_STATUS_ETC_ERROR;
		}

		if( NUPDATE_STATUS_STILL_ACTIVE == status )
		{
			_sleep(10);
			continue;
		}

		switch( status )
		{
		case NUPDATE_STATUS_SUCCESS:
			return TRUE;


		case NUPDATE_STATUS_ERROR:
			return FALSE;

		default:
			return FALSE;
		}
	}

	return TRUE;
}

BOOL UpgradeInstallModule::Install()
{
	GetCallback()->OnStart();
	CALLBACK_FINISHED_STATUS status = CALLBACK_FINISHED_SUCCESS;
	ERRORINFO *errorInfo = NULL;

	NAVERAPP_INSTALL_STATUS install_status = GetInstallStatusWithGuid(GetGuid());
	if( NotInstalled == install_status )
	{
		status = CALLBACK_FINISHED_ERROR;

	}else if ( OldVersion == install_status )
	{
		int code = 0;
		CString msg;

		if (FALSE == RunUpgrader(GetGuid(), m_execute_option, &code, &msg) )
		{
			status = CALLBACK_FINISHED_ERROR;
			errorInfo = MakeErrorInfo(code, (const TCHAR *)msg);
		}
	}

	SetModuleStatus(ModuleStatusFINISHED);
	GetCallback()->OnFinished(status, errorInfo);
	return TRUE;
}